<?php

/**
 * @file
 * Provides payment method for Realex Redirect.
 */

class pay_realex_method_redirect extends pay_method_direct {
  var $realex_merchid = '';
  var $realex_secret = '';
  var $realex_account = '';
  var $auto_settle_flag = 1;
  var $realex_varref = '';
  var $realex_referring_url = '';
  var $realex_test_mode = 0;
  var $payment_type = 'realex_redirect';

  function available_currencies() {
    return array(
      'EUR', // Euro.
      'GBP', // Pound Sterling.
      'USD', // U.S. Dollar.
      'SEK', // Swedish Krona.
      'CHF', // Swiss Franc.
      'HKD', // Hong Kong Dollar.
      'JPY', // Japanese Yen.
    );
  }

  function form(&$form, &$form_state) {
    global $user;
    parent::form($form, $form_state);

    if (isset($this->realex_test_mode) && $this->realex_test_mode) {
      drupal_set_message(t('The @name payment method is in test mode. This transaction will not be fully processed.', array('@name' => $this->title())), 'warning');
    }
  }

  function form_submit($form, &$form_state) {
    // Find our transaction object provided by pay_form::form_submit().
    if (isset($form_state['pay_activity']) && $form_state['pay_activity']) {
      foreach ($form_state['pay_activity'] as $key => $activity) {
        if ($activity->pmid == $this->pmid) {
          // Load the activity as the 'current' one.
          $this->activity = pay_activity_load($activity->paid);
          $this->__construct($this->activity->data);
          $this->activity->action = $this->pay_form_action;

          // Effect a redirect to the direct payment gateway.
          if ($this->activity->action && $this->activity->action != 'pending') {
            unset($form_state['storage']);
            $form_state['rebuild'] = FALSE;
            $form_state['redirect'] = $this->direct_request();
            break;
          }
        }
      }
    }
  }

  function direct_request() {
    global $user;
    if ($user->uid == 0) {
      drupal_session_start();
    }
    $token = drupal_get_token($this->activity->paid);
    return array(
      'pay/realex/redirect/' . $this->activity->paid,
      array('query' => array('token' => $token)),
    );
  }

  function complete_action($values = array()) {

    $pxid = $this->activity->pxid;
    $paid = $this->activity->paid;

    // Prepare the SHA1 hash.
    $timestamp = strftime('%Y%m%d%H%M%S');
    mt_srand((double) microtime() * 1000000);
    $orderid = $pxid . '-' . $timestamp;
    $amount = (int)($this->total() * 100);
    // TODO: the currency is hardcoded.
    $currency = 'EUR';
    $hash = $timestamp . '.' . $this->realex_merchid . '.' . $orderid . '.' . $amount . '.' . $currency;
    $sha1hash = sha1($hash);
    $hash = $sha1hash . '.' . $this->realex_secret;
    $sha1hash = sha1($hash);

    // Prepare the Realex billing code field.
    $street_one = isset($this->billto['street1']) ? check_plain($this->billto['street1']) : '';
    $street_two = isset($this->billto['street2']) ? check_plain($this->billto['street2']) : '';
    $street_three = isset($this->billto['street3']) ? check_plain($this->billto['street3']) : '';
    $city = isset($this->billto['city']) ? check_plain($this->billto['city']) : '';
    $state = isset($this->billto['state']) ? check_plain($this->billto['state']) : '';
    $postcode = isset($this->billto['zip']) ? check_plain($this->billto['zip']) : '';
    $country = isset($this->billto['country']) ? check_plain($this->billto['country']) : '';
    $country = (!empty($country) && strlen($country) == 2) ? pay_realex_country_numeric_code($country) : ''; 
    $phone = isset($this->billto['phone']) ? check_plain($this->billto['phone']) : '';
    $email = isset($this->mail) ? check_plain($this->mail) : '';
    $zip_numbers = preg_replace('/[^0-9]/', '', $postcode);
    $street_numbers = preg_replace('/[^0-9]/', '', $street_one);
    $zip_postal_code = $zip_numbers . '|' . $street_numbers;
    $zip_postal_code = ($zip_postal_code == '|') ? '' : $zip_postal_code;


    $token = drupal_get_token($this->activity->paid);

    // Set up data array.
    $values = array(
      'MERCHANT_RESPONSE_URL' => url('pay-realex/realex-redirect-response', array('absolute' => TRUE)),
      'MERCHANT_ID'           => $this->realex_merchid,
      'ACCOUNT'               => $this->realex_account,
      'ORDER_ID'              => $orderid,
      'AMOUNT'                => $amount,
      'CURRENCY'              => $currency,
      'TIMESTAMP'             => $timestamp,
      'SHA1HASH'              => $sha1hash,
      'AUTO_SETTLE_FLAG'      => $this->auto_settle_flag,
      'VAR_REF'               => $this->realex_varref,
      'RETURN_TSS'            => 1,
      'PAY_PXID'              => $pxid,
      'PAY_PAID'              => $paid,
      //'HPP_TEMPLATE_TYPE'     => 'LIGHTBOX',
      'HPP_ORIGIN'            => url('<front>', array('absolute' => TRUE)),
      'gateway_url'           => $this->gateway_url(),

      'HPP_VERSION' => 2,
      'COMMENT1' => 'drupalpay7',
      'HPP_CUSTOMER_EMAIL'    => $email,
      'HPP_CUSTOMER_PHONENUMBER_MOBILE' => $phone,

      // Billing information.
      'HPP_BILLING_STREET1'   => $street_one,
      'HPP_BILLING_STREET2'   => $street_two,
      'HPP_BILLING_STREET3'   => $street_three,
      'HPP_BILLING_CITY'      => $city,
      'HPP_BILLING_STATE'     => $state,
      'HPP_BILLING_POSTALCODE' => $postcode,
      'HPP_BILLING_COUNTRY'   => $country,

      // No shipping information - set to empty.
      'HPP_ADDRESS_MATCH_INDICATOR' => 'FALSE',
      'HPP_SHIPPING_STREET1'    => '',
      'HPP_SHIPPING_STREET2'    => '',
      'HPP_SHIPPING_STREET3'    => '',
      'HPP_SHIPPING_CITY'       => '',
      'HPP_SHIPPING_STATE'      => '',
      'HPP_SHIPPING_POSTALCODE' => '',
      'HPP_SHIPPING_COUNTRY'    => '',
    );
    // Pass through source webform id for later use.
    if (arg(0) == 'node') {
      $values['source_webform_node'] = arg(1);
    }
    if (!empty($this->billto['country'])) {
      $values['BILLING_CO'] = $this->billto['country'];
    }
    if (!empty($zip_postal_code)) {
      $values['BILLING_CODE'] = $zip_postal_code;
    }

    parent::complete_action($values);

    return;
  }

  function gateway_url() {
    $payment_gateway = $this->realex_test_mode ? 'hpp.sandbox' : 'hpp';
    $payment_gateway = 'https://' . $payment_gateway . '.realexpayments.com/pay';
    return $payment_gateway;
  }


  function direct_response($response = array()) {
    $this->activity->result = FALSE;

    $this->activity->data[] = $response;

    // Verify HASH.
    $merchid = $this->realex_merchid;
    $secret = $this->realex_secret;
    $orderid = base64_decode($response['ORDER_ID']);
    $resultcode = base64_decode($response['RESULT']);
    $message = base64_decode($response['MESSAGE']);
    if ($resultcode != '00') {
      $this->activity->result = FALSE;
      $this->activity->data = array(
        'error' => $message,
        'error_code' => $resultcode,
        'print_error' => 1,
      );
      return $this->activity->result;
    }
    $pasref = isset($response['PASREF']) ? base64_decode($response['PASREF']) : '';
    $authcode = isset($response['AUTHCODE']) ? base64_decode($response['AUTHCODE']) : '';
    $timestamp = base64_decode($response['TIMESTAMP']);

    // This digital signature should correspond to the one Realex Payments returns
    // and can therefore be used to verify the message Realex Payments sends back.
    $tmp = "$timestamp.$merchid.$orderid.$resultcode.$message.$pasref.$authcode";
    $sha1hash = sha1($tmp);
    $tmp = "$sha1hash.$secret";
    $sha1hash = sha1($tmp);

    $success = FALSE;

    // Check to see if hashes match or not
    if ($sha1hash != base64_decode($response['SHA1HASH'])) {
      watchdog('pay_realex', 'Hash match failed for order @order_id.', array('@order_id' => $orderid));
      drupal_set_message(t("Hashes don't match - response not authenticated!"), 'error');
      $message = t('Payment error, please try again later');
      drupal_set_message($message, 'error');
      $this->activity->data = array(
        'error' => $error,
        'print_error' => 1,
      );
      $this->activity->result = FALSE;
      return $this->activity->result;
    }

    unset($response['SHA1HASH']);
    $amount = base64_decode($response['AMOUNT']);

    $this->activity->result = TRUE;
    $this->activity->action = 'complete';
    $this->activity->transaction_total = $amount;
    $this->activity->total = $amount;
    $this->activity->identifier = $authcode;
    $this->activity->data = array(
      'timestamp' => $timestamp,
      'response_string' => $message,
      'result_code' => $resultcode,
      'approval_code' => $authcode,
      'cvnresult' => base64_decode($response['CVNRESULT']),
      'pasref' => $pasref,
      'batchid' => base64_decode($response['BATCHID']),
      'transaction_total' => $amount,
    );
    if (!empty($response['AVSPOSTCODERESULT'])) {
      $this->activity->data['avspostcoderesult'] = base64_decode($response['AVSPOSTCODERESULT']);
    }
    if (!empty($response['AVSADDRESSRESULT'])) {
      $this->activity->data['avsaddressresult'] = base64_decode($response['AVSADDRESSRESULT']);
    }
    if (!empty($response['CAVV'])) {
      $this->activity->data['cavv_response'] = base64_decode($response['CAVV']);
    }
    if (!empty($response['ECI'])) {
      $this->activity->data['eci_response'] = base64_decode($response['ECI']);
    }
    if (!empty($response['XID'])) {
      $this->activity->data['xid_response'] = base64_decode($response['XID']);
    }


    return $this->activity->result;
  }

  function settings_form(&$form, &$form_state) {
    parent::settings_form($form, $form_state);
    $group = $this->handler();

    $form[$group]['realex_redirect']['#type'] = 'fieldset';
    $form[$group]['realex_redirect']['#collapsible'] = FALSE;
    $form[$group]['realex_redirect']['#title'] = t('Realex Redirect settings');
    $form[$group]['realex_redirect']['#group'] = $group;

    $form[$group]['realex_redirect']['realex_merchid'] = array(
      '#type' => 'textfield',
      '#title' => t('Realex Merchant ID'),
      '#default_value' => $this->realex_merchid,
      '#required' => TRUE,
      '#parents' => array($group, 'realex_merchid'),
    );
    $form[$group]['realex_redirect']['realex_account'] = array(      '#type' => 'textfield',
      '#title' => t('Realex Account'),
      '#default_value' => $this->realex_account,
      '#required' => TRUE,
      '#parents' => array($group, 'realex_account'),
    );
    $form[$group]['realex_redirect']['realex_secret'] = array(
      '#type' => 'textfield',
      '#title' => t('Shared Secret'),
      '#default_value' => $this->realex_secret,
      '#required' => TRUE,
      '#parents' => array($group, 'realex_secret'),
    );
    $form[$group]['realex_redirect']['realex_referring_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Referring URL'),
      '#description' => t('The absolute url to the page on your site from which the Realex Payments gateway will be referred by. You will need to give this url to Realex Payments when they set up your account.'),
      '#default_value' => $this->realex_referring_url,
      '#required' => TRUE,
      '#parents' => array($group, 'realex_referring_url'),
    );

    $form[$group]['realex_redirect']['auto_settle_flag'] = array(
      '#type' => 'select',
      '#title' => t('Auto settle flag'),
      '#options' => array(1 => t('Enabled'), 0 => t('Disabled')),
      '#default_value' => $this->auto_settle_flag,
      '#required' => TRUE,
      '#parents' => array($group, 'auto_settle_flag'),
    );
    $form[$group]['realex_redirect']['realex_varref'] = array(
      '#type' => 'textfield',
      '#title' => t('Variable reference'),
      '#default_value' => $this->realex_varref,
      '#required' => FALSE,
      '#parents' => array($group, 'realex_varref'),
    );
    $form[$group]['realex_redirect']['realex_test_mode'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use Realex sandbox'),
      '#default_value' => $this->realex_test_mode,
      '#required' => FALSE,
      '#parents' => array($group, 'realex_test_mode'),
    );
  }


  function submit_realex_form($form_id, $data) {
    $form_state['values'] = $data['data'];
    $form_state['build_info']['args'] = array(&$data);
    drupal_form_submit($form_id, $form_state);
  }

}
